--Dynamic list of itemTypes
local itemTypeNames = {}

local inventoryToSortHeader = {
   	[INVENTORY_BACKPACK] 			= ZO_PlayerInventorySortBy,
   	[INVENTORY_BANK] 				= ZO_PlayerBankSortBy,
   	[INVENTORY_GUILD_BANK] 			= ZO_GuildBankSortBy,
   	[INVENTORY_CRAFT_BAG] 			= ZO_CraftBagSortBy,
	[INVENTORY_HOUSE_BANK] 			= ZO_HouseBankSortBy,
}
local craftingInventoryData = {
	--Crafting
	["smithingRefinementListView"]      = {
		["sortHeaderControl"]	= ZO_SmithingTopLevelRefinementPanelInventorySortBy,
		["inventory"]			= SMITHING.refinementPanel.inventory,
	},
	["smithingDeconstructionListView"]  = {
		["sortHeaderControl"]	= ZO_SmithingTopLevelDeconstructionPanelInventorySortBy,
		["inventory"]			= SMITHING.deconstructionPanel.inventory,

	},
	["smithingImprovementListView"]     = {
		["sortHeaderControl"]	= ZO_SmithingTopLevelImprovementPanelInventorySortBy,
		["inventory"]			= SMITHING.improvementPanel.inventory,
	},
	["enchantingTableListView"]         = {
		["sortHeaderControl"]	= ZO_EnchantingTopLevelInventorySortBy,
		["inventory"]			= ENCHANTING.inventory,
	},
	--Retrait
	["retraitListView"]         		= {
		["sortHeaderControl"]	= ZO_RetraitStation_KeyboardTopLevelRetraitPanelInventorySortBy,
		["inventory"]			= ZO_RETRAIT_STATION_KEYBOARD.retraitPanel.inventory,
	},
}

CraftingMaterialLevelDisplay = {
	name			= "CraftingMaterialLevelDisplay",
    versionString 	= "2.20",
	fontStyles = {
		"none",
		"outline",
		"thin-outline",
		"thick-outline",
		"shadow",
		"soft-shadow-thin",
		"soft-shadow-thick",
	},
	defaultSavedVariablesAccount = {
		--1: Account wide, 2: Each character
		saveMode = 2, -- Each character
	},
	defaultSavedVariables = {
		--provisioning = true,
		--provisioningFlavor = true,
		alchemy = true,
		enchanting = true,
		jewelryCrafting = true,
		showLevelsInInventoryLists = true,
		showLevelsInCraftingInventoryLists = true,
		blacksmithingInventoryList = true,
		clothingInventoryList = true,
		enchantingInventoryList = true,
		woodworkingInventoryList = true,
		jewelryCraftingInventoryList = true,
        --provisioningInventoryList = true,
        font = {
        	family 	= "Univers 55",
            size    = 14,
            style	= "none",
            color	= {
            	["r"] = 1,
                ["g"] = 1,
                ["b"] = 1,
                ["a"] = 1
            }
        },
		levelSortHeader = true,
        normalInventoryList = true,
        inventoryOffset = -100,
        lootOffset = -20,
		levelHeaderOffsetX = 50,
		normalCraftingInventoryList = true,
		itemTypeLabels = {
			[ITEMTYPE_ADDITIVE] = true,
			[ITEMTYPE_ARMOR] = true,
			[ITEMTYPE_ARMOR_BOOSTER] = true,
			[ITEMTYPE_ARMOR_TRAIT] = true,
			[ITEMTYPE_BLACKSMITHING_BOOSTER] = true,
			[ITEMTYPE_BLACKSMITHING_MATERIAL] = true,
			[ITEMTYPE_BLACKSMITHING_RAW_MATERIAL] = true,
			[ITEMTYPE_CLOTHIER_BOOSTER] = true,
			[ITEMTYPE_CLOTHIER_MATERIAL] = true,
			[ITEMTYPE_CLOTHIER_RAW_MATERIAL] = true,
			[ITEMTYPE_CONTAINER] = true,
			[ITEMTYPE_DRINK] = true,
			[ITEMTYPE_ENCHANTING_RUNE_ASPECT] = true,
			[ITEMTYPE_ENCHANTING_RUNE_ESSENCE] = true,
			[ITEMTYPE_ENCHANTING_RUNE_POTENCY] = true,
			[ITEMTYPE_ENCHANTMENT_BOOSTER] = true,
			[ITEMTYPE_FLAVORING] = true,
			[ITEMTYPE_FOOD] = true,
			[ITEMTYPE_FURNISHING] = true,
			[ITEMTYPE_FURNISHING_MATERIAL] = true,
			[ITEMTYPE_INGREDIENT] = true,
			[ITEMTYPE_POISON] = true,
			[ITEMTYPE_POISON_BASE] = true,
			[ITEMTYPE_POTION] = true,
			[ITEMTYPE_POTION_BASE] = true,
			[ITEMTYPE_RAW_MATERIAL] = true,
			[ITEMTYPE_REAGENT] = true,
			[ITEMTYPE_RECIPE] = true,
			[ITEMTYPE_SOUL_GEM] = true,
			[ITEMTYPE_TOOL] = true,
			[ITEMTYPE_WEAPON] = true,
			[ITEMTYPE_WEAPON_BOOSTER] = true,
			[ITEMTYPE_WEAPON_TRAIT] = true,
			[ITEMTYPE_WOODWORKING_BOOSTER] = true,
			[ITEMTYPE_WOODWORKING_MATERIAL] = true,
			[ITEMTYPE_WOODWORKING_RAW_MATERIAL] = true,
		},
	},
	currentInventoryRows = {}
}

--Libraries
CraftingMaterialLevelDisplay.LAM = LibAddonMenu2
if CraftingMaterialLevelDisplay.LAM == nil and LibStub then CraftingMaterialLevelDisplay.LAM = LibStub:GetLibrary("LibAddonMenu-2.0") end
CraftingMaterialLevelDisplay.LMP = LibMediaProvider
if CraftingMaterialLevelDisplay.LMP == nil and LibStub then CraftingMaterialLevelDisplay.LMP = LibStub:GetLibrary('LibMediaProvider-1.0') end

--Inventories
CraftingMaterialLevelDisplay.craftingInventoryData = craftingInventoryData

function CraftingMaterialLevelDisplay.GetItemIdFromLink(itemLink)
	return GetItemLinkItemId(itemLink)
end

function CraftingMaterialLevelDisplay.GetItemIdFromBagAndSlot(bagId, slotIndex)
    local itemLink = GetItemLink(bagId, slotIndex)
    return GetItemLinkItemId(itemLink)
end

function CraftingMaterialLevelDisplay.UpdateSortHeader(headerCtrl, isInventoryHeader)
	if headerCtrl == nil then
--d("[UpdateSortHeader] All inventories")
		--All inventories
		if isInventoryHeader then
			for _, sortParent in pairs(inventoryToSortHeader) do
				local levelHeader = WINDOW_MANAGER:GetControlByName(sortParent:GetName().."Level", "")
				if levelHeader then
--d(">> Header: " .. levelHeader:GetName())
					levelHeader:SetHidden(not CraftingMaterialLevelDisplay.savedVariables.levelSortHeader)
					levelHeader:SetMouseEnabled(CraftingMaterialLevelDisplay.savedVariables.levelSortHeader)
				end
			end
		else
			--All crafting inventories
			for _, craftingInventoryDataEntry in pairs(craftingInventoryData) do
				local sortParent = craftingInventoryDataEntry["sortHeaderControl"]
				if sortParent ~= nil then
					local levelHeader = WINDOW_MANAGER:GetControlByName(sortParent:GetName().."Level", "")
					if levelHeader then
--d(">> Crafting header: " .. levelHeader:GetName())
						levelHeader:SetHidden(not CraftingMaterialLevelDisplay.savedVariables.levelSortHeaderCrafting)
						levelHeader:SetMouseEnabled(CraftingMaterialLevelDisplay.savedVariables.levelSortHeaderCrafting)
					end
				end
			end

		end
	else
		--d("[UpdateSortHeader] " .. headerCtrl:GetName() .. ", isInventoryHeader: " .. tostring(isInventoryHeader))
		--Only a special inventory
		local hiddenFlagVar
		if isInventoryHeader then
			hiddenFlagVar = CraftingMaterialLevelDisplay.savedVariables.levelSortHeader
		else
			hiddenFlagVar = CraftingMaterialLevelDisplay.savedVariables.levelSortHeaderCrafting
		end
		headerCtrl:SetHidden(not hiddenFlagVar)
		headerCtrl:SetMouseEnabled(hiddenFlagVar)
	end
end

local function GetItemMinMaxLevel(tradeSkillType, itemType, itemId)
	local itemLevelMin = -1
    local itemLevelMax = -1
-- d("[GetItemMinMaxLevel] Start - tradeSkillType: " .. tostring(tradeSkillType) .. ", itemType: " .. tostring(itemType) .. ", itemId: " .. tostring(itemId))
    if   tradeSkillType == nil or itemType == nil or itemId == nil
	  or tradeSkillType == 0 or itemType == 0 or itemId == 0 then
    	return itemLevelMin, itemLevelMax
    end

    local levelStr  = ""

    if tradeSkillType == CRAFTING_TYPE_CLOTHIER then
-- d("[GetItemMinMaxLevel] Clothier")
		if ClothingMaterials[itemId] and ClothingMaterials[itemId].level ~= nil then
			levelStr = ClothingMaterials[itemId].level
		end

    elseif tradeSkillType == CRAFTING_TYPE_BLACKSMITHING then
-- d("[GetItemMinMaxLevel] Blacksmith")
		if BlacksmithingMaterials[itemId] and BlacksmithingMaterials[itemId].level ~= nil then
			levelStr = BlacksmithingMaterials[itemId].level
		end

    elseif tradeSkillType == CRAFTING_TYPE_WOODWORKING then
-- d("[GetItemMinMaxLevel] Woodworking")
		if WoodworkingMaterials[itemId] and WoodworkingMaterials[itemId].level ~= nil then
			levelStr = WoodworkingMaterials[itemId].level
		end

    elseif tradeSkillType == CRAFTING_TYPE_ENCHANTING then
--d("[GetItemMinMaxLevel] Enchanting")
		if EnchantingMaterials[itemId] and EnchantingMaterials[itemId].level ~= nil then
			levelStr = EnchantingMaterials[itemId].level
		end
	--[[
    elseif tradeSkillType == CRAFTING_TYPE_PROVISIONING then
-- d("[GetItemMinMaxLevel] Provision")
		if ProvisioningMaterials[itemId] and ProvisioningMaterials[itemId].level ~= nil and ProvisioningMaterials[itemId].level ~= "-1" then
			levelStr = ProvisioningMaterials[itemId].level
		end
	]]--
	elseif tradeSkillType == CRAFTING_TYPE_JEWELRYCRAFTING then
		--d("[GetItemMinMaxLevel] Jewelry")
		if JewelryCraftingMaterials[itemId] and JewelryCraftingMaterials[itemId].level ~= nil then
			levelStr = JewelryCraftingMaterials[itemId].level
		end
    end

-- d("[GetItemMinMaxLevel] LevelStr: " .. levelStr)

	--Split the level string into min and max values
    if levelStr ~= "" and levelStr ~= "-1" then
        --Remove spaces from levelStr
        levelStr = levelStr:gsub("%s+", "")
-- d("[GetItemMinMaxLevel] LevelStr without spaces: " .. levelStr)
		--Set offset for search start
        local untilHereStart
        local untilHereEnd
		local itemLevelMinStr
		local itemLevelMaxStr
		--Find the "-" character
        untilHereStart, untilHereEnd = string.find (levelStr, "-")
        --Was the "-" character found?
        if untilHereStart ~= nil then
--d("[GetItemMinMaxLevel] - found at " .. tostring(untilHereStart))
			--"-" character found -> Level range "from - to"
	    	itemLevelMinStr = string.sub(levelStr, 1, untilHereStart-1)
	        itemLevelMaxStr = string.sub(levelStr, untilHereStart+1)
        else
--d("[GetItemMinMaxLevel] - not found")
			--"-" character not found -> Only one level
	    	itemLevelMinStr = levelStr
	        itemLevelMaxStr = itemLevelMinStr
        end
--d("[GetItemMinMaxLevel] minStr: " .. itemLevelMinStr .. ", maxStr: " .. itemLevelMaxStr)

        --Change string values to numbers (or try it, as levels could be strings like "V2" for veteran ranks)
        itemLevelMin = tonumber(itemLevelMinStr)
        itemLevelMax = tonumber(itemLevelMaxStr)
        local vetRankLevel
		--The maximum player level for normal levels (< Veteran ranks)
        local maxNormalLevel = 50
        --Check if values are numbers
        if itemLevelMin == nil then
			vetRankLevel = 0
			--No number -> Veteran rank
            --Convert veteran rank level to number
            vetRankLevel = tonumber(string.sub(itemLevelMinStr, 3))
--d("[GetItemMinMaxLevel] vetLevelMin: " .. tostring(vetRankLevel))
            itemLevelMin = maxNormalLevel + vetRankLevel
        end
        if itemLevelMax == nil then
			vetRankLevel = 0
			--No number -> Veteran rank
            --Convert veteran rank level to number
            vetRankLevel = tonumber(string.sub(itemLevelMaxStr, 3))
--d("[GetItemMinMaxLevel] vetLevelMax: " .. tostring(vetRankLevel))
            itemLevelMax = maxNormalLevel + vetRankLevel
        end
    end

--d("[GetItemMinMaxLevel] Min: " .. tostring(itemLevelMin) .. ", Max: " .. tostring(itemLevelMax))

	return itemLevelMin, itemLevelMax
end

function CraftingMaterialLevelDisplay.updateLabels(labelCtrl, isLootWindow)
	isLootWindow = isLootWindow or false
	local offset = 0
    if isLootWindow then
    	offset = CraftingMaterialLevelDisplay.savedVariables.lootOffset
    else
    	offset = CraftingMaterialLevelDisplay.savedVariables.inventoryOffset
    end

    local color = CraftingMaterialLevelDisplay.savedVariables.font.color
    local fontPath = CraftingMaterialLevelDisplay.LMP:Fetch('font', CraftingMaterialLevelDisplay.savedVariables.font.family)
	local fontString = string.format('%s|%u|%s', fontPath, CraftingMaterialLevelDisplay.savedVariables.font.size, CraftingMaterialLevelDisplay.savedVariables.font.style)
	--local fontString = "ZoFontGame"
	if labelCtrl == nil then
	    for _,labelControl in pairs(CraftingMaterialLevelDisplay.currentInventoryRows) do
	        labelControl:SetFont(fontString)
	        labelControl:SetColor(color.r, color.g, color.b, color.a)
            labelControl:ClearAnchors()
            labelControl:SetAnchor(RIGHT, labelControl:GetParent(), RIGHT, offset)
	    end
    else
	    labelCtrl:SetFont(fontString)
	    labelCtrl:SetColor(color.r, color.g, color.b, color.a)
        labelCtrl:ClearAnchors()
        labelCtrl:SetAnchor(RIGHT, labelCtrl:GetParent(), RIGHT, offset)
    end

end

local function InitializeSavedVariables()
	--Build the itemType tables
	for i=ITEMTYPE_MIN_VALUE, ITEMTYPE_MAX_VALUE, 1 do
		local itemTypeNameNumber = _G["SI_ITEMTYPE" .. i]
		if itemTypeNameNumber ~= nil then
			itemTypeNames[i] = ZO_CachedStrFormat("<<C:1>>", GetString(itemTypeNameNumber))
		end
		CraftingMaterialLevelDisplay.defaultSavedVariables.itemTypeLabels[i] = CraftingMaterialLevelDisplay.defaultSavedVariables.itemTypeLabels[i] or false
	end
	--Get the SAvedVariables
    CraftingMaterialLevelDisplay.savedVariables = ZO_SavedVars:New("CMLD_SavedVariables", 1, nil, CraftingMaterialLevelDisplay.defaultSavedVariables)
	CraftingMaterialLevelDisplay.itemTypeNames = itemTypeNames
end

local function orderByItemLevel(data1, data2, currentSortOrder)
	--Comparison result constants
	local IS_LESS_THAN = -1
	local IS_EQUAL_TO = 0
	local IS_GREATER_THAN = 1

	--Tiebraker function for equal levels: Sort by level first, then by name to keep the list sorted alphabetically
	local function tieBrakeByName(p_data1, p_data2, sortOrder)
		local name1 = zo_strlower(tostring(p_data1.name))
		local name2 = zo_strlower(tostring(p_data2.name))
		local compRes
		if name1 ~= nil and name2 ~= nil then
			if name1 > name2 then
				compRes = IS_GREATER_THAN
			elseif name1 < name2 then
				compRes = IS_LESS_THAN
			end
		else
			return nil
		end
		--[[
		if sortOrder == ZO_SORT_ORDER_UP then
			return compRes == IS_LESS_THAN
		end
		return compRes == IS_GREATER_THAN
		]]
		--The name sorting should always be done alphabetically ascending
		return compRes == IS_LESS_THAN
	end
	--Variables
    local minLevel1
    local minLevel2
    local maxLevel1
    local maxLevel2
    local vetLevel
	--Get bag and slotIndex for item 1 and comparison item 2
	local bagId1, slotIndex1 = data1.bagId, data1.slotIndex
	local bagId2, slotIndex2 = data2.bagId, data2.slotIndex
	--Get the itemlink, itemlveles etc. as a number value (veteran ranks will be calculated as level 50 + vet rank to number)
	--Data for item 1
	local link   = GetItemLink(bagId1, slotIndex1)
    local itemId = CraftingMaterialLevelDisplay.GetItemIdFromLink(link)
    local tradeSkillType, itemType = GetItemCraftingInfo(bagId1, slotIndex1)
	minLevel1, maxLevel1 = GetItemMinMaxLevel(tradeSkillType, itemType, itemId)
	if minLevel1 == -1 and maxLevel1 == -1 then
		minLevel1 = GetItemLinkRequiredLevel(link)
		vetLevel  = GetItemLinkRequiredVeteranRank(link)
		if vetLevel > 0 then
			minLevel1 = minLevel1 + vetLevel
		end
	    if minLevel1 == 1 then
	    	minLevel1 = GetItemLevel(bagId1, slotIndex1)
	    end
    end
	--Data for comparison item 2
    link = GetItemLink(bagId2, slotIndex2)
    itemId = CraftingMaterialLevelDisplay.GetItemIdFromLink(link)
    tradeSkillType, itemType = GetItemCraftingInfo(bagId2, slotIndex2)
	minLevel2, maxLevel2 = GetItemMinMaxLevel(tradeSkillType, itemType, itemId)
	if minLevel2 == -1 and maxLevel2 == -1 then
		minLevel2 = GetItemLinkRequiredLevel(link)
		vetLevel  = GetItemLinkRequiredVeteranRank(link)
		if vetLevel > 0 then
			minLevel2 = minLevel2 + vetLevel
		end
	    if minLevel2 == 1 then
	    	minLevel2 = GetItemLevel(bagId2, slotIndex2)
	    end
    end

	--The variables for the comparisonResult of the level and the resulting sortReturn value for this sorting function
	local compareResult
	local sortReturn
	local sortByNameTiebraker = false

	--Todo: For debugging only:
	--Begin: debugging variables
	local sortedThisWay = 0
	local sortRetStr = ""
	--End: debugging variables

	--Now compare the item1 level with the comparison item 2 level
	--  All levels are equal. Sort by name only
	if minLevel1 == maxLevel1 == minLevel2 == maxLevel2 then
		compareResult = IS_EQUAL_TO
		sortedThisWay = 1
	--  Min and max levels of item1 are equal to item2.
	--	Sort by name
	elseif minLevel1 == minLevel2 and maxLevel1 == maxLevel2 then
		compareResult = IS_EQUAL_TO
		sortedThisWay = 2

	--  Levels of item1 are equal and levels of item2 are equal.
	--	Sort by min level + name afterwards
	elseif minLevel1 == maxLevel1 and minLevel2 == maxLevel2 then
		if minLevel1 < minLevel2 then
			sortedThisWay = 3
			compareResult = IS_LESS_THAN
		elseif minLevel1 > minLevel2 then
			sortedThisWay = 4
			compareResult = IS_GREATER_THAN
		else
			sortedThisWay = 5
			compareResult = IS_EQUAL_TO
		end

	-- MaxLevels unequal both -1
	elseif maxLevel1 ~= -1 and maxLevel2 ~= -1 then
		--MinLevel and MaxLevel comparison of item1 < item2
		if minLevel1 < minLevel2 and maxLevel1 < maxLevel2 then
			sortedThisWay = 6
			compareResult = IS_LESS_THAN
		elseif minLevel1 > minLevel2 or maxLevel1 > maxLevel2 then
			sortedThisWay = 7
			compareResult = IS_GREATER_THAN
		else
			sortedThisWay = 8
			compareResult = IS_EQUAL_TO
		end

	-- Max levels equal both -1
	elseif maxLevel1 == -1 and maxLevel2 == -1 then
		--Min level comparison
		if minLevel1 < minLevel2 then
			sortedThisWay = 9
			compareResult = IS_LESS_THAN
		elseif minLevel1 > minLevel2 then
			sortedThisWay = 10
			compareResult = IS_GREATER_THAN
		else
			sortedThisWay = 11
			compareResult = IS_EQUAL_TO
		end

	-- Max level2 equals -1
	elseif maxLevel1 ~= -1 and maxLevel2 == -1 then
		--Min level comparison & minlevel2 comparison against maxlevel1
		if minLevel1 < minLevel2 then
			sortedThisWay = 12
			compareResult = IS_LESS_THAN
		elseif minLevel1 > minLevel2 then
			sortedThisWay = 13
			compareResult = IS_GREATER_THAN
		else
			sortedThisWay = 14
			compareResult = IS_EQUAL_TO
		end

	-- Max level1 equals -1 but maxlevel2 not
	elseif maxLevel1 == -1 and maxLevel2 ~= -1 then
		--Min level comparison & minlevel1 comparison against maxlevel2
		if minLevel1 < minLevel2 then
			sortedThisWay = 15
			compareResult = IS_LESS_THAN
		elseif minLevel1 > minLevel2 then
			sortedThisWay = 16
			compareResult = IS_GREATER_THAN
		else
			sortedThisWay = 17
			compareResult = IS_EQUAL_TO
		end

	--All other comparisons
	else
		--Minlevel comparison and maxlevel comparison
		if minLevel1 < minLevel2 and maxLevel1 < maxLevel2 then
			sortedThisWay = 18
			compareResult = IS_LESS_THAN
		--Minlevel comparison
		elseif minLevel1 < minLevel2 then
			sortedThisWay = 19
			compareResult = IS_LESS_THAN
		--Maxlevel comparison
		elseif maxLevel1 < maxLevel2 then
			compareResult = IS_LESS_THAN
		-----------------------------------------------------------
		--Minlevel comparison and maxlevel comparison
		elseif minLevel1 > minLevel2 and maxLevel1 > maxLevel2 then
			sortedThisWay = 20
			compareResult = IS_GREATER_THAN
		--Minlevel comparison
		elseif minLevel1 > minLevel2 then
			sortedThisWay = 21
			compareResult = IS_GREATER_THAN
		--Maxlevel comparison
		elseif maxLevel1 > maxLevel2 then
			sortedThisWay = 22
			compareResult = IS_GREATER_THAN
		-----------------------------------------------------------
		else
			sortedThisWay = 23
			compareResult = IS_EQUAL_TO
		end
	end

	--Check the results now and get true or false value according to the used sortOrder
	if compareResult == IS_EQUAL_TO then
		--Comparison was equal, so return false and do not sort this item. Just check the name tiebrake afterwards
		sortReturn = false
		sortByNameTiebraker = true
		sortRetStr = "equal"
	else
		if currentSortOrder == ZO_SORT_ORDER_UP then
			sortReturn = compareResult == IS_LESS_THAN
		else
			sortReturn = compareResult == IS_GREATER_THAN
		end
	end
	--Sort by name as a tiebraker (for equal levels) too?
	if sortByNameTiebraker then
		sortReturn = tieBrakeByName(data1, data2, currentSortOrder)
	end

--[[
	--Todo: Comment, as only needed for debug message
	if sortRetStr == "" then
		if sortReturn == true then
			sortRetStr = "before"
		else
			sortRetStr = "after"
		end
	end
d(">Sorted (".. tostring(sortReturn) ..") " .. GetItemLink(bagId1, slotIndex1) .." sorted \"" .. tostring(sortRetStr) .."\" " .. GetItemLink(bagId2, slotIndex2) .." [minLevel1: " .. minLevel1 .. ", maxLevel1: " .. maxLevel1 .. " / minLevel2: " .. minLevel2.. ", maxLevel2: " .. maxLevel2 .. ", sortByNameToo: " ..tostring(sortByNameTiebraker) .. "] ->   " .. tostring(sortedThisWay))
]]

	--Return false or true for the comparison of the 2 dataEntries
	-->false:	Move the compared item data1 before the item with data2
	-->true:	Move the compared item data1 after the item with data2
	return sortReturn
end

--Enable that a "key" can be a function to sort the items
local function initializeCustomInventorySortFn(inventory, isInventory)
	local function sortInventoryItems(entry1, entry2)
		local sortKey
		local sortOrder
		if isInventory then
			sortKey = inventory.currentSortKey
			sortOrder = inventory.currentSortOrder
		else
			sortKey = inventory.sortKey
			sortOrder = inventory.sortOrder
		end
        local res
        if type(sortKey) == "function" then
			--No sort order difference in this function anymore. WAs moved to sorting function itsself!
			res = sortKey(entry1.data, entry2.data, sortOrder)
        else
            local sortKeys
			if isInventory then
				sortKeys = ZO_Inventory_GetDefaultHeaderSortKeys()
			else
				sortKeys =
				{
					slotIndex = { isNumeric = true },
					stackCount = { tiebreaker = "slotIndex", isNumeric = true },
					name = { tiebreaker = "quality" },
					quality = { tiebreaker = "stackCount", isNumeric = true },
					statusSortOrder = { tiebreaker = "name", isNumeric = true },
					stackSellPrice = { tiebreaker = "name", isNumeric = true },
					traitInformationSortOrder = { tiebreaker = "name", isNumeric = true },
					custom = { tiebreaker = "name"},
				}
			end
			res = ZO_TableOrderingFunction(entry1.data, entry2.data, sortKey, sortKeys, sortOrder)
        end
        return res
    end
	if isInventory then
		inventory.sortFn = sortInventoryItems
	else
		inventory.sortFunction = sortInventoryItems
	end
end

-- call from EVENT_ADD_ON_LOADED
local function HookInventorySortByLevel(inv, isInvHeader)
	isInvHeader = isInvHeader or false
--d("[CMLD] HookInventorySortByLevel, inv: " .. tostring(inv) .. ", isInvHeader: " ..tostring(isInvHeader))
	--if inventoryToSortHeader[inv] and inventoryToSortHeader[inv]:GetName() then
	--	d(">parentName: " .. inventoryToSortHeader[inv]:GetName())
	--end
    local invSortByParentControl
	local inventory
	if isInvHeader then
		invSortByParentControl = inventoryToSortHeader[inv]
		inventory = PLAYER_INVENTORY.inventories[inv]
	else
		local craftingInventoryDataEntry = craftingInventoryData[inv]
		if craftingInventoryDataEntry == nil then return false end
		invSortByParentControl = craftingInventoryDataEntry["sortHeaderControl"]
		inventory = craftingInventoryDataEntry["inventory"]
	end
	if invSortByParentControl == nil then return false end
	if inventory == nil then return false end
    local nameHeader = invSortByParentControl:GetNamedChild("Name")
    local levelHeader = CreateControlFromVirtual("$(parent)Level", invSortByParentControl, "ZO_SortHeader")
    local levelHeaderAnchorOffsetX = CraftingMaterialLevelDisplay.savedVariables.levelHeaderOffsetX
	levelHeader:SetAnchor(RIGHT, nameHeader, RIGHT, levelHeaderAnchorOffsetX, 0)
    levelHeader:SetDimensions(80, 20)
    --Set an own sorting function as "key" (instead of a string), which will be made functional via function "initializeCustomInventorySortFn(inventory)"
	ZO_SortHeader_Initialize(levelHeader, "Level", orderByItemLevel, ZO_SORT_ORDER_UP, TEXT_ALIGN_CENTER, "ZoFontHeader")
	--Support the own sorting function at the "key"
	initializeCustomInventorySortFn(inventory, isInvHeader)
    inventory.sortHeaders:AddHeader(levelHeader)
--d("HookInventorySortByLevel--->")
	CraftingMaterialLevelDisplay.UpdateSortHeader(levelHeader, isInvHeader)
end

local function onLoad(_, name)
	if name ~= CraftingMaterialLevelDisplay.name then return end
	EVENT_MANAGER:UnregisterForEvent(CraftingMaterialLevelDisplay.name, EVENT_ADD_ON_LOADED);
	InitializeSavedVariables()
	CraftingMaterialLevelDisplay.BuildAddonMenu()
	CraftingMaterialLevelDisplay.HookTooltips()
	CraftingMaterialLevelDisplay.HookInventoryLists()
	--Add sort by level to inventory
	HookInventorySortByLevel(INVENTORY_BACKPACK, true)
	HookInventorySortByLevel(INVENTORY_BANK, true)
	HookInventorySortByLevel(INVENTORY_GUILD_BANK, true)
	HookInventorySortByLevel(INVENTORY_HOUSE_BANK, true)
	--Does the player have access to the craft bag?
	if HasCraftBagAccess() then
		HookInventorySortByLevel(INVENTORY_CRAFT_BAG, true)
	end
	--Add sort by level to crafting inventory
	for craftingInventoryDataName, _ in pairs(craftingInventoryData) do
		HookInventorySortByLevel(craftingInventoryDataName, false)
	end
end

function CMLD_GetItemLevel(tradeSkillType, itemType, itemId)
-- d("[CMLD] Start - tradeSkillType: " .. tostring(tradeSkillType) .. ", itemType: " .. tostring(itemType) .. ", itemId: " .. tostring(itemId))
	local itemLevelMin, itemLevelMax = GetItemMinMaxLevel(tradeSkillType, itemType, itemId)
--d("[CMLD_GetItemLevel] Min: " .. tostring(itemLevelMin) .. ", Max: " .. tostring(itemLevelMax))
	return itemLevelMin, itemLevelMax
end

EVENT_MANAGER:RegisterForEvent(CraftingMaterialLevelDisplay.name, EVENT_ADD_ON_LOADED, onLoad)
