--The champion rank levels
--[[
	[60] = "CP10",
	[70] = "CP20",
	[80] = "CP30",
	[90] = "CP40",
	[100] = "CP50",
	[110] = "CP60",
	[120] = "CP70",
	[130] = "CP80",
	[140] = "CP90",
	[150] = "CP100",
	[160] = "CP110",
	[170] = "CP120",
	[180] = "CP130",
	[190] = "CP140",
	[200] = "CP150",
	[210] = "CP160",
]]
local maxNormalLevel = GetMaxLevel()
local minVeteranLevel = 10
local maxVeteranLevel = GetChampionPointsPlayerProgressionCap()
local normalToVeteranIncrement = 10
local veteranPrefixStr = "CP"
local veteranLevelToLevelStr = {}
for i = minVeteranLevel, maxVeteranLevel, normalToVeteranIncrement do
    local veteranLevelStr = veteranPrefixStr .. tostring(i)
    if veteranLevelStr ~= veteranPrefixStr then
        veteranLevelToLevelStr[maxNormalLevel + i] = veteranLevelStr
    end
end

local function getLabelForInventoryRowControl(row, itemId)
    local label = CraftingMaterialLevelDisplay.currentInventoryRows[row:GetName()]
    if not label then
        label = WINDOW_MANAGER:CreateControl(row:GetName() .. "CMLD", row, CT_LABEL)
        CraftingMaterialLevelDisplay.currentInventoryRows[row:GetName()] = label
    end
    label:SetText(itemId)
    --label:SetFont("ZoFontGame")
	CraftingMaterialLevelDisplay.updateLabels(label)
    label:ClearAnchors()
    label:SetHidden(true)
    return label
end

local function IsTheRowRectangular(rowControl)
    return rowControl:GetWidth() / rowControl:GetHeight() > 1.5
end

local function DisplayTheLabel(rowControl, labelControl, offset, text)
    if IsTheRowRectangular(rowControl) then
        labelControl:SetText(text)
        labelControl:SetAnchor(RIGHT, rowControl, RIGHT, offset)
        labelControl:SetHidden(false)
    end
end

--If item got a level but the itemId was not added to the list of crafting/other items to show the level for:
--Get the item's level now and display it
local function ShowOtherItemsLevel(rowControl, label, offset, itemId, itemLink, itemLevel)
--d("[CMLD]ShowOtherItemsLevel, itemId: " ..tostring(itemId) .. ", itemLink: " .. itemLink .. ", itemLevel: " ..tostring(itemLevel))
    if rowControl == nil or label == nil or offset == nil or itemId == nil or itemLink == nil then return nil end
    local levelStr = ""
    --Check if item type should show an label
    local itemType = GetItemLinkItemType(itemLink)
    --Local variable for allowed itemTypes which should be shown at the inventory item (from the LAM settings)
    local allowedItemTypesForLabel = CraftingMaterialLevelDisplay.savedVariables.itemTypeLabels
    if allowedItemTypesForLabel[itemType] then
        local level = GetItemLinkRequiredLevel(itemLink)
        if level == 1 and itemLevel ~= -1 then
            level = itemLevel
        end
        local vetLevel = GetItemLinkRequiredVeteranRank(itemLink)
        if vetLevel > 0 then
            level = level + vetLevel
        end
        --Output the level label
        --Veteran ranks?
        if level > maxNormalLevel then
            levelStr = veteranLevelToLevelStr[level]
        else
            levelStr = tostring(level)
        end
        --Error prevention
        if levelStr == nil or levelStr == "" then
            return nil
        else
            DisplayTheLabel(rowControl, label, offset, "["..levelStr.."]")
        end
    end
end

local function AddCraftingMaterialLevelToInventoryRow(rowControl, tradeSkillType, itemType, itemId, offset, itemLink, itemLevel)
	local label = getLabelForInventoryRowControl(rowControl, itemId)

    if tradeSkillType == CRAFTING_TYPE_CLOTHIER then
        if CraftingMaterialLevelDisplay.savedVariables.clothingInventoryList then
            if ClothingMaterials[itemId] and ClothingMaterials[itemId].level ~= nil then
                DisplayTheLabel(rowControl, label, offset, "["..ClothingMaterials[itemId].level.."]")
            else
                if not CraftingMaterialLevelDisplay.savedVariables.normalCraftingInventoryList then return nil end
                ShowOtherItemsLevel(rowControl, label, offset, itemId, itemLink, itemLevel)
            end
        end

    elseif tradeSkillType == CRAFTING_TYPE_BLACKSMITHING then
        if CraftingMaterialLevelDisplay.savedVariables.blacksmithingInventoryList then
            if BlacksmithingMaterials[itemId] and BlacksmithingMaterials[itemId].level ~= nil then
                DisplayTheLabel(rowControl, label, offset, "["..BlacksmithingMaterials[itemId].level.."]")
            else
                if not CraftingMaterialLevelDisplay.savedVariables.normalCraftingInventoryList then return nil end
                ShowOtherItemsLevel(rowControl, label, offset, itemId, itemLink, itemLevel)
            end
        end

    elseif tradeSkillType == CRAFTING_TYPE_WOODWORKING then
        if CraftingMaterialLevelDisplay.savedVariables.woodworkingInventoryList then
            if WoodworkingMaterials[itemId] and WoodworkingMaterials[itemId].level ~= nil then
                DisplayTheLabel(rowControl, label, offset, "["..WoodworkingMaterials[itemId].level.."]")
            else
                if not CraftingMaterialLevelDisplay.savedVariables.normalCraftingInventoryList then return nil end
                ShowOtherItemsLevel(rowControl, label, offset, itemId, itemLink, itemLevel)
            end
        end

    elseif tradeSkillType == CRAFTING_TYPE_JEWELRYCRAFTING then
        if CraftingMaterialLevelDisplay.savedVariables.jewelryCraftingInventoryList then
            if JewelryCraftingMaterials[itemId] and JewelryCraftingMaterials[itemId].level ~= nil then
                DisplayTheLabel(rowControl, label, offset, "["..JewelryCraftingMaterials[itemId].level.."]")
            else
                if not CraftingMaterialLevelDisplay.savedVariables.normalCraftingInventoryList then return nil end
                ShowOtherItemsLevel(rowControl, label, offset, itemId, itemLink, itemLevel)
            end
        end

    elseif tradeSkillType == CRAFTING_TYPE_ENCHANTING
            and itemType ~= ITEMTYPE_GLYPH_ARMOR
            and itemType ~= ITEMTYPE_GLYPH_JEWELRY
            and itemType ~= ITEMTYPE_GLYPH_WEAPON then
        -- Does not need to account for the created Glyphs, just the runes
        if CraftingMaterialLevelDisplay.savedVariables.enchantingInventoryList then
            if EnchantingMaterials[itemId] and EnchantingMaterials[itemId].level ~= nil then
                DisplayTheLabel(rowControl, label, offset, "["..EnchantingMaterials[itemId].level.."]")
            else
                if not CraftingMaterialLevelDisplay.savedVariables.normalCraftingInventoryList then return nil end
                ShowOtherItemsLevel(rowControl, label, offset, itemId, itemLink, itemLevel)
            end
        end

        --[[
        elseif tradeSkillType == CRAFTING_TYPE_PROVISIONING then
            -- Does not need to account for the created Glyphs, just the runes
            if CraftingMaterialLevelDisplay.savedVariables.provisioningInventoryList then
                if ProvisioningMaterials[itemId] and ProvisioningMaterials[itemId].level ~= nil and ProvisioningMaterials[itemId].level ~= "-1" then
                    DisplayTheLabel(rowControl, label, offset, "["..ProvisioningMaterials[itemId].level.."]")
                end
            end
        ]]--

        --No crafting material but normal
    else
        if not CraftingMaterialLevelDisplay.savedVariables.normalInventoryList then return nil end
        ShowOtherItemsLevel(rowControl, label, offset, itemId, itemLink, itemLevel)
    end
end

-- See PLAYER_INVENTORY["bagToInventoryType"] for the mappings from bag constants to inventory IDs
local inventoryLists = {
    ["backpackListView"]        = PLAYER_INVENTORY.inventories[INVENTORY_BACKPACK].listView,
    ["bankListView"]            = PLAYER_INVENTORY.inventories[INVENTORY_BANK].listView,
    ["guildBankListView"]       = PLAYER_INVENTORY.inventories[INVENTORY_GUILD_BANK].listView,
	["craftBagListView"]        = PLAYER_INVENTORY.inventories[INVENTORY_CRAFT_BAG].listView,
    --House bank
    ["houseBankListView"]       = PLAYER_INVENTORY.inventories[INVENTORY_HOUSE_BANK].listView,
}
local craftingInventoryLists = {}
local craftingInventoryListWasBuild = false

local function hookInventoryLists(inventoryListsToUse)
    for _, inventoryList in pairs(inventoryListsToUse) do
        if inventoryList and inventoryList.dataTypes and inventoryList.dataTypes[1] then
            --Check if player got craft bag access, or if the inventory hook is not for the craft bag
            local isCraftBagInvList = (inventoryList == PLAYER_INVENTORY.inventories[INVENTORY_CRAFT_BAG].listView) or false
            if not isCraftBagInvList or (isCraftBagInvList and HasCraftBagAccess()) then
                --Hook the inventory list
                local existingCallbackFunction = inventoryList.dataTypes[1].setupCallback
                inventoryList.dataTypes[1].setupCallback = function(rowControl, slot)
                    existingCallbackFunction(rowControl, slot)
                    local slotIndex = "slotIndex" and slot["slotIndex"] or nil
                    local bagId = slot["bagId"]
                    local itemId = CraftingMaterialLevelDisplay.GetItemIdFromBagAndSlot(bagId, slotIndex)
                    local tradeSkillType, itemType = GetItemCraftingInfo(bagId, slotIndex)
                    local itemLink = GetItemLink(bagId, slotIndex)
                    local itemLevel = GetItemLevel(bagId, slotIndex)
                    AddCraftingMaterialLevelToInventoryRow(rowControl, tradeSkillType, itemType, itemId, CraftingMaterialLevelDisplay.savedVariables.inventoryOffset, itemLink, itemLevel)
                end
            end
        end
    end
end

local function HookNormalInventoryLists()
    hookInventoryLists(inventoryLists)
end

local function HookCraftingInventoryLists()
    --Build the craftingInventoryLists
    if craftingInventoryLists == nil or not craftingInventoryListWasBuild then
        local craftingInventoryData = CraftingMaterialLevelDisplay.craftingInventoryData
        if craftingInventoryData ~= nil then
            craftingInventoryLists = {}
            for craftingInventoryName, craftingInventoryDataEntry in pairs(craftingInventoryData) do
                local craftingInventoryList = craftingInventoryDataEntry["inventory"].list
                if craftingInventoryList ~= nil then
                    craftingInventoryLists[craftingInventoryName] = craftingInventoryList
                end
            end
            craftingInventoryListWasBuild = true
        end
    end
    if craftingInventoryLists ~= nil then
        CraftingMaterialLevelDisplay.craftingInventoryLists = craftingInventoryLists
        hookInventoryLists(craftingInventoryLists)
    end
end

local function HookLootWindowInventoryList()
    local existingCallbackFunction = LOOT_WINDOW.list.dataTypes[1].setupCallback
    LOOT_WINDOW.list.dataTypes[1].setupCallback = function(rowControl, slot)
        existingCallbackFunction(rowControl, slot)
        local itemLink = GetLootItemLink(slot.lootId)
        local itemId = CraftingMaterialLevelDisplay.GetItemIdFromLink(itemLink)
        local tradeSkillType = GetItemLinkCraftingSkillType(itemLink)
        AddCraftingMaterialLevelToInventoryRow(rowControl, tradeSkillType, nil, itemId, CraftingMaterialLevelDisplay.savedVariables.lootOffset, itemLink, -1)
    end
end

function CraftingMaterialLevelDisplay.HookInventoryLists()
    if CraftingMaterialLevelDisplay.savedVariables.showLevelsInInventoryLists then
        HookNormalInventoryLists()
        HookLootWindowInventoryList()
    end
    if CraftingMaterialLevelDisplay.savedVariables.showLevelsInCraftingInventoryLists then
        HookCraftingInventoryLists()
    end
end
