CraftingMaterialLevelDisplay = CraftingMaterialLevelDisplay or {}

local function HideCurrentInventoryRows()
    for _,labelControl in pairs(CraftingMaterialLevelDisplay.currentInventoryRows) do
        labelControl:SetHidden(true)
    end
end

function CraftingMaterialLevelDisplay.BuildAddonMenu()

    local LAM = CraftingMaterialLevelDisplay.LAM

    CraftingMaterialLevelDisplay.savedVariablesAccount = ZO_SavedVars:NewAccountWide("CMLD_SavedVariables_Account", 1, nil, CraftingMaterialLevelDisplay.defaultSavedVariablesAccount, GetWorldName(), "AllAccounts")
    if CraftingMaterialLevelDisplay.savedVariablesAccount.saveMode == 2 then
        CraftingMaterialLevelDisplay.savedVariables = ZO_SavedVars:NewCharacterIdSettings("CMLD_SavedVariables", 1, nil, CraftingMaterialLevelDisplay.defaultSavedVariables, GetWorldName())
    else
        CraftingMaterialLevelDisplay.savedVariables = ZO_SavedVars:NewAccountWide("CMLD_SavedVariables", 1, nil, CraftingMaterialLevelDisplay.defaultSavedVariables, GetWorldName())
    end
    local settings = CraftingMaterialLevelDisplay.savedVariables
    local defaults = CraftingMaterialLevelDisplay.defaultSavedVariables

    local saveModeEntries = {
        [1] = "Account wide (all accounts the same)",
        [2] = "Different for each character",
    }
    local saveModeEntriesValues = {
        [1] = 1,
        [2] = 2,
    }

    local panelData = {
        type = "panel",
        name = "Crafting Material Level Display",
        displayName = ZO_HIGHLIGHT_TEXT:Colorize("Crafting Material Level Display"),
        author = "Marihk, Ayantir & Baertram",
        version = CraftingMaterialLevelDisplay.versionString,
        slashCommand = "/cmld",
        registerForRefresh = true,
        registerForDefaults = true,
        website = "https://www.esoui.com/downloads/fileinfo.php?id=758",
    }
    LAM:RegisterAddonPanel(CraftingMaterialLevelDisplay.name .. "ControlPanel", panelData)

    --Function to create a LAM control
    local function CreateControl(ref, name, tooltip, data, disabledChecks, getFunc, setFunc, defaultSettings, warning, scrollable)
        scrollable = scrollable or false
        if ref ~= nil then
            if string.find(ref, "CMLD_LAM_", 1)  ~= 1 then
                data.reference = "CMLD_LAM_" .. ref
            else
                data.reference = ref
            end
        end
        if data.type ~= "description" then
            data.name = name
            if data.type ~= "header" and data.type ~= "submenu" then
                data.tooltip = tooltip
                if data.type ~= "button" then
                    data.getFunc = getFunc
                    data.setFunc = setFunc
                    data.default = defaultSettings
                else
                    data.func = setFunc
                end
                if disabledChecks ~= nil then
                    data.disabled = disabledChecks
                end
                data.scrollable = scrollable
                data.warning = warning
            end
        end
        return data
    end

    local itemTypesSubMenuControls = {}
    local function buildItemTypeSubMenu()
        local itemTypeNames = CraftingMaterialLevelDisplay.itemTypeNames
        if itemTypeNames ~= nil and #itemTypeNames > 0 then
            for itemType, itemTypeName in ipairs(itemTypeNames) do
                --Build a checkbox for each itemType and place all the checkboxes into a submenu
                local ref = "CMLD_LAM_cb_ItemType" .. tostring(itemType)
                local name = itemTypeName
                local tooltip = itemTypeName
                local data = { type = "checkbox", width = "half" }
                local disabledFunc = function() return not settings.normalInventoryList and not settings.normalCraftingInventoryList end
                local getFunc
                local setFunc
                local defaultSettings
                getFunc = function() return settings.itemTypeLabels[itemType] end
                setFunc = function(value) settings.itemTypeLabels[itemType] = value end
                defaultSettings = defaults.itemTypeLabels[itemType]
                --Create the dropdownbox now
                local createdItemTypeCB = CreateControl(ref, name, tooltip, data, disabledFunc, getFunc, setFunc, defaultSettings, nil)
                if createdItemTypeCB ~= nil then
                    table.insert(itemTypesSubMenuControls, createdItemTypeCB)
                end
            end
        end
    end
    buildItemTypeSubMenu()

    --[[
    local fontPreview = function(panel)
        if panel == CMLDSettingsPanel then
            previewLabel = WINDOW_MANAGER:CreateControl(nil, panel.controlsToRefresh[10], CT_LABEL)
            previewLabel:SetAnchor(RIGHT, panel.controlsToRefresh[10].dropdown:GetControl(), LEFT, -10, 0)
            previewLabel:SetText("[CP100-CP140]")
            previewLabel:SetHidden(false)
            CraftingMaterialLevelDisplay.updateLabels(previewLabel)
            CALLBACK_MANAGER:UnregisterCallback("LAM-RefreshPanel", fontPreview)
        end
    end

    CALLBACK_MANAGER:RegisterCallback("LAM-PanelControlsCreated", fontPreview)

    ]]--

    local optionsTable = {

        {
            type = 'dropdown',
            name = "Save mode",
            choices = saveModeEntries,
            choicesValues = saveModeEntriesValues,
            getFunc = function() return CraftingMaterialLevelDisplay.savedVariablesAccount.saveMode end,
            setFunc = function(value) CraftingMaterialLevelDisplay.savedVariablesAccount.saveMode = value
            end,
            default = CraftingMaterialLevelDisplay.defaultSavedVariablesAccount.saveMode,
            requiresReload = true,
        },

        --[[
        {
            type = "checkbox",
            name = "Show Provisioning tooltips",
            getFunc = function() return settings.provisioning end,
            setFunc = function(newValue) settings.provisioning = newValue end,
            default = defaults.provisioning,
        },
        {
            type = "checkbox",
            name = "Use Flavor text instead of Tier 2/3 text",
            getFunc = function() return settings.provisioningFlavor end,
            setFunc = function(newValue) settings.provisioningFlavor = newValue end,
            default = defaults.provisioningFlavor,
        },

        ]]--

        {
            type = "checkbox",
            name = "Show Alchemy tooltips",
            getFunc = function() return settings.alchemy end,
            setFunc = function(newValue) settings.alchemy = newValue end,
            default = defaults.alchemy,
        },
        {
            type = "checkbox",
            name = "Show Enchanting tooltips",
            getFunc = function() return settings.enchanting end,
            setFunc = function(newValue) settings.enchanting = newValue end,
            default = defaults.enchanting,
        },
        {
            type = "checkbox",
            name = "Show Jewelry tooltips",
            getFunc = function() return settings.jewelryCrafting end,
            setFunc = function(newValue) settings.jewelryCrafting = newValue end,
            default = defaults.jewelryCrafting,
        },
        {
            type = "checkbox",
            name = "Show levels in inventory lists",
            getFunc = function() return settings.showLevelsInInventoryLists end,
            setFunc = function(newValue) settings.showLevelsInInventoryLists = newValue end,
            default = defaults.showLevelsInInventoryLists,
            warning = "Requires /reloadui to take effect, prevents the inventory list hook code from even executing, useful if errors happen",
            requiresReload = true,
        },
        {
            type = "checkbox",
            name = "Show levels in crafting inventory lists",
            getFunc = function() return settings.showLevelsInCraftingInventoryLists end,
            setFunc = function(newValue) settings.showLevelsInCraftingInventoryLists = newValue end,
            default = defaults.showLevelsInCraftingInventoryLists,
            warning = "Requires /reloadui to take effect, prevents the crafting inventory list hook code from even executing, useful if errors happen",
            requiresReload = true,
        },
        {
            type = "checkbox",
            name = "Show Enchanting levels in inventory",
            getFunc = function() return settings.enchantingInventoryList end,
            setFunc = function(newValue)
                settings.enchantingInventoryList = newValue
                if newValue == false then
                    HideCurrentInventoryRows()
                end
            end,
            default = defaults.enchantingInventoryList,
        },
        {
            type = "checkbox",
            name = "Show Blacksmithing levels in inventory",
            getFunc = function() return settings.blacksmithingInventoryList end,
            setFunc = function(newValue)
                settings.blacksmithingInventoryList = newValue
                if newValue == false then
                    HideCurrentInventoryRows()
                end
            end,
            default = defaults.blacksmithingInventoryList,
        },
        {
            type = "checkbox",
            name = "Show Clothing levels in inventory",
            getFunc = function() return settings.clothingInventoryList end,
            setFunc = function(newValue)
                settings.clothingInventoryList = newValue
                if newValue == false then
                    HideCurrentInventoryRows()
                end
            end,
            default = defaults.clothingInventoryList,
        },
        {
            type = "checkbox",
            name = "Show Woodworking levels in inventory",
            getFunc = function() return settings.woodworkingInventoryList end,
            setFunc = function(newValue)
                settings.woodworkingInventoryList = newValue
                if newValue == false then
                    HideCurrentInventoryRows()
                end
            end,
            default = defaults.woodworkingInventoryList,
        },

        --[[

        {
            type = "checkbox",
            name = "Show Provisioning levels in inventory",
            getFunc = function() return settings.provisioningInventoryList end,
            setFunc = function(newValue)
                settings.provisioningInventoryList = newValue
                if newValue == false then
                    HideCurrentInventoryRows()
                end
            end,
            default = defaults.provisioningInventoryList,
        },

        ]]--

        {
            type = "checkbox",
            name = "Show Jewelry crafting levels in inventory",
            getFunc = function() return settings.jewelryCraftingInventoryList end,
            setFunc = function(newValue)
                settings.jewelryCraftingInventoryList = newValue
                if newValue == false then
                    HideCurrentInventoryRows()
                end
            end,
            default = defaults.jewelryCraftingInventoryList,
        },
        {
            type = "checkbox",
            name = "Show normal items levels in inventory",
            tooltip = "Also show item levels for weapons, armor, etc.",
            getFunc = function() return settings.normalInventoryList end,
            setFunc = function(newValue)
                settings.normalInventoryList = newValue
                if newValue == false then
                    HideCurrentInventoryRows()
                end
            end,
            default = settings.normalInventoryList,
        },
        {
            type = "checkbox",
            name = "Show normal items levels in crafting inventory",
            tooltip = "Also show item levels for other crafting materials",
            getFunc = function() return settings.normalCraftingInventoryList end,
            setFunc = function(newValue)
                settings.normalCraftingInventoryList = newValue
                if newValue == false then
                    HideCurrentInventoryRows()
                end
            end,
            default = settings.normalCraftingInventoryList,
        },
        {
            type = 		"submenu",
            name = 		"Allowed item types of normal items",
            controls = 	itemTypesSubMenuControls,
        },
        {
            type = 'dropdown',
            name = "Font family",
            choices = CraftingMaterialLevelDisplay.LMP:List('font'),
            getFunc = function() return settings.font.family end,
            setFunc = function(value) settings.font.family = value
                --CraftingMaterialLevelDisplay.updateLabels(previewLabel)
                CraftingMaterialLevelDisplay.updateLabels()
            end,
            default = settings.font.family,
        },
        {
            type = "slider",
            name = "Font size",
            min = 10,
            max = 48,
            getFunc = function() return settings.font.size end,
            setFunc = function(size) settings.font.size = size
                --CraftingMaterialLevelDisplay.updateLabels(previewLabel)
                CraftingMaterialLevelDisplay.updateLabels()
            end,
            default = settings.font.size,
        },
        {
            type = 'dropdown',
            name = "Font style",
            choices = CraftingMaterialLevelDisplay.fontStyles,
            getFunc = function() return settings.font.style end,
            setFunc = function(value) settings.font.style = value
                --CraftingMaterialLevelDisplay.updateLabels(previewLabel)
                CraftingMaterialLevelDisplay.updateLabels()
            end,
            default = settings.font.style,
        },
        {
            type = "colorpicker",
            name = "Font color",
            getFunc = function() return settings.font.color.r, settings.font.color.g, settings.font.color.b, settings.font.color.a end,
            setFunc = function(r,g,b,a) settings.font.color = {["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a}
                --CraftingMaterialLevelDisplay.updateLabels(previewLabel)
                CraftingMaterialLevelDisplay.updateLabels()
            end,
            default = settings.font.color,
        },
        {
            type = "checkbox",
            name = "Add 'Level' sort header in inventory",
            getFunc = function() return settings.levelSortHeader end,
            setFunc = function(newValue)
                settings.levelSortHeader = newValue
                CraftingMaterialLevelDisplay.UpdateSortHeader(nil, true)
            end,
            default = settings.levelSortHeader,
        },
        {
            type = "checkbox",
            name = "Add 'Level' sort header in crafting inventory",
            getFunc = function() return settings.levelSortHeaderCrafting end,
            setFunc = function(newValue)
                settings.levelSortHeaderCrafting = newValue
                CraftingMaterialLevelDisplay.UpdateSortHeader(nil, false)
            end,
            default = settings.levelSortHeaderCrafting,
        },
        {
            type = "slider",
            name = "Offset of level header in inventory",
            min = -300,
            max = 200,
            getFunc = function() return settings.levelHeaderOffsetX end,
            setFunc = function(size) settings.levelHeaderOffsetX = size
            end,
            requiresReload = true,
            default = settings.levelHeaderOffsetX,
        },
        {
            type = "slider",
            name = "Offset of level in inventory",
            min = -540,
            max = 0,
            getFunc = function() return settings.inventoryOffset end,
            setFunc = function(size) settings.inventoryOffset = size
                CraftingMaterialLevelDisplay.updateLabels()
            end,
            default = settings.inventoryOffset,
        },
        {
            type = "slider",
            name = "Offset of level in loot window",
            min = -540,
            max = 0,
            getFunc = function() return settings.lootOffset end,
            setFunc = function(size) settings.lootOffset = size
                CraftingMaterialLevelDisplay.updateLabels(nil, true)
            end,
            default = settings.lootOffset,
        },
    }
    LAM:RegisterOptionControls(CraftingMaterialLevelDisplay.name .. "ControlPanel", optionsTable)
end